$title Homework 5: Problem 1

$offuellist offuelxref offsymlist offsymxref
option limrow=0, limcol=0, solprint=off;


set iter/iter1*iter10/,
    nodes /1*2000/;

alias (nodes,i,j,k);
set arcs(i,j);

scalar
       global_density / 0.001 /
       local_density  / 0.05 /
       local_range    / 50 /
       arc_length     Length of Each Arc / 1 /
       successful     Number of Successful Runs / 0 /
       averageLength  / 0 /
       proportionOfSuccess
       expected_outdegree;


parameter source(iter) Source Nodes,
          sink(iter) Sink Nodes,
          flow(iter, i, j) Flow On Arcs by Iteration,
          supply(nodes),
          avg_outdegree(iter) Average Number of Arcs Leaving Each Node,
          arcs_out(nodes),
          std_dev(iter) Standard Deviation of Outgoing Arcs,
          shortest_path(iter) Shortest Path Length By Iteration;

positive variable f(i,j);

free variable shortestPathLength;

equations balance(i), objective;

balance(i)..
        sum(arcs(i,k), f(arcs)) - sum(arcs(j,i), f(arcs)) =e= supply(i);
objective..
        shortestPathLength =e= sum(arcs, f(arcs));

model prob5/balance, objective/;
option lp = cplex;
prob5.optfile = 1;

option seed=25671;
loop(iter,
*        create arcs
*        create global connections
         arcs(i, j) = no;
         arcs(i, j)$(uniform(0,1) < global_density/2) = yes;
*        create local connections
         arcs(i, j)$(abs(ord(i) - ord(j)) <= 50 and uniform(0, 1) < local_density/2) = yes;
         arcs(j, i)$(arcs(i, j)) = yes;
*        remove self connections
         arcs(i,i) = no;

*        select source and destination nodes
         source(iter) = floor(uniform(1, card(nodes) + .9999));
         sink(iter) = floor(uniform(1, card(nodes) + .9999));

         while(sink(iter) = source(iter),
                 sink(iter) = round(uniform(1, card(nodes)));
         );

         supply(i)$(ord(i) = source(iter)) = 1;
         supply(i)$(ord(i) = sink(iter)) = -1;
         display supply;

*calculate average outdegree
         arcs_out(i) = sum(j$arcs(i, j), 1);
         avg_outdegree(iter) = (sum(i, arcs_out(i)) / card(nodes));

*calculate standard deviation
         std_dev(iter) = sqrt(sum(nodes, sqr(arcs_out(nodes) - avg_outdegree(iter))) /
                              (card(nodes) - 1));

*solve for shortest path
         solve prob5 using lp minimizing shortestPathLength;
         if(prob5.modelstat < 2,
                 successful = successful+1;
                 averageLength = averageLength + shortestPathLength.l;
         );
*copy parameters for easier display later
         flow(iter, i, j) = f.l(i, j);
         shortest_path(iter) = shortestPathLength.l;

         f.l(i, j) = 0;
         supply(i) = 0;
);

averageLength = (averageLength / successful)$(successful > 0);
proportionOfSuccess = successful / card(iter);
*expected density = all nodes except yourself * global density + local nodes * local density
*note this is a "high" upperbound since nodes near the ends of the array cant connect to the
*full amount of local nodes
expected_outdegree = (card(nodes) - 1) * global_density + (local_range * 2) * local_density;

*display options
option averageLength:8;
option source:0:0:1;
option sink:0:0:1;
option flow:0:0:1;
option shortest_path:0:0:1;

*display values
display source;
display sink;
display flow;
display shortest_path;
display "Local Density: ", local_density;
display "Global Density: ", global_density;
display "Expected Outdegree: ", expected_outdegree;
display avg_outdegree;
display std_dev;
display "Proportion Of Successful Paths: ", proportionOfSuccess;
display "Average Path Length Of Successful Paths: ", averageLength;