$title Homework 8: Problem 1

$offuellist offuelxref offsymlist offsymxref
option limrow=0, limcol=0, solprint=off;
option mip = cplex;

$ontext
 The total amount of radiation delivered to a particular region is obtained by
 summing weight(beam)*(dose delivered by beam to region) over all beams.

 1. Suppose that the physicians are allowed to use at most three beams, and that
 the maximum weight that can be applied to any beam is 3. They wish to ensure
 that at least 80 units of radiation are delivered to each tumor region, while
 minimizing the total dose to the normal regions. Formulate a GAMS model and
 solve to find the optimal treatment plan.
$offtext

set beams / beam1*beam6 /,
    regions / normal1*normal3, tumor1*tumor3 /,
    normalRegions(regions),
    tumorRegions(regions);

alias (beams, b);
alias (regions, r);

normalRegions(regions) = yes$(ord(regions) <= 3);
tumorRegions(regions)  = yes$(not normalRegions(regions));

scalar maxBeams maximum number of different beams that can be used / 3 /,
       maxWeight maximum number of times each beam can be used     / 3 /,
       minRadiation min. amount radiation be be delivered to each tumor / 80 /;

table dose(beams,regions) dose delivered by each beam to each region
          normal1  normal2  normal3  tumor1  tumor2  tumor3
beam1        6        3        8       20      12       6
beam2       12       10        6       18      15       8
beam3        9        8       13       13      10      17
beam4        4       12       12        6      18      16
beam5        9        4       11       13       6      14
beam6        8        7        7       10      10      10
;

positive variable weight(beams) weight to use each beam;
positive variables totalRad(regions) total amount of radiation by region;
binary variable active(beams) determines whether or not to use each beam;

free variable totalNormalRad total amount of radiation delivered to normal tissue;

equations EQ_totalRad(regions) total amount of radiation delivered to each region,
          EQ_minRadiation(regions),
          EQ_maxBeams,
          EQ_maxWeight(beams),
          EQ_totalNormalRad;

* summing weight(beam)*(dose delivered by beam to region) over all beams
EQ_totalRad(regions)..
         totalRad(regions) =e= sum(b, weight(b) * dose(b, regions));

* ensure minimium amount of radiation is delivered to the tumor regions
EQ_minRadiation(tumorRegions)..
         totalRad(tumorRegions) =g= minRadiation;

* limit the number of beams that can be used
EQ_maxBeams..
         sum(b, active(b)) =l= maxBeams;

* ensures that each beam used can only be used to a certain weight
EQ_maxWeight(beams)..
         weight(beams) =l= maxWeight * active(beams);

EQ_totalNormalRad..
         totalNormalRad =e= sum(normalRegions, totalRad(normalRegions));

model prob8 /all/;
prob8.optcr = 0;

solve prob8 using mip minimizing totalNormalRad;

option weight:0;
display weight.l;
display totalRad.l;
display totalNormalRad.l;