$title Homework 8: Problem 2

$offuellist offuelxref offsymlist offsymxref
option limrow=0, limcol=0, solprint=off;
option mip = cplex;

$ontext
 The total amount of radiation delivered to a particular region is obtained by
 summing weight(beam)*(dose delivered by beam to region) over all beams.

2. Suppose now that the physicians wish to use a more sophisticated treatment
 plan. They wish to deliver a dose of between 90 and 95 units to each tumor
 region. Any dose of more than 95 units incurs a penalty of 5 times the amount
 of violation, while any dose of less than 90 units incurs a penalty of 10 times
 the amount of violation. They are also prepared to "sacrifice" one of the
 normal regions. That is, the dose delivered to one of the normal regions is
 allowed to exceed 45, while the dose delivered to the other two normal regions
 must remain below 45.

 Given that they wish to minimize the objective formed by adding the penalties,
 and given that they can still use only at most 3 beams with weight at most 3
 each, use GAMS to determine the optimal treatment plan.
$offtext

set beams / beam1*beam6 /,
    regions / normal1*normal3, tumor1*tumor3 /,
    normalRegions(regions),
    tumorRegions(regions);

alias (beams, b);
alias (regions, r);

normalRegions(regions) = yes$(ord(regions) <= 3);
tumorRegions(regions)  = yes$(not normalRegions(regions));

scalar bigM /1000/,
       maxBeams maximum number of different beams that can be used / 3 /,
       maxWeight maximum number of times each beam can be used / 3 /,
       overPenalty penalty(per unit) of going over wanted rad. of tumor / 5 /,
       underPenalty penalty(per unit) of going under wanted rad. of tumor / 10 /,
       maxNormRad largest amount of rad. to deliver to a normal region / 45 /,
       minTumorRad least amount of radiation to deliver to each tumor   / 90 /,
       maxTumorRad largest amount of radiation to deliver to each tumor / 95 /,
       normalLose  acceptable amount of normal regions to lose / 1 /;

table dose(beams,regions) dose delivered by each beam to each region
          normal1  normal2  normal3  tumor1  tumor2  tumor3
beam1        6        3        8       20      12       6
beam2       12       10        6       18      15       8
beam3        9        8       13       13      10      17
beam4        4       12       12        6      18      16
beam5        9        4       11       13       6      14
beam6        8        7        7       10      10      10
;

*integer variable weight(beams) weight to use each beam;
positive variable weight(beams) weight to use each beam;

positive variables totalRad(regions) total amount of radiation by region,
                   penalty(regions) penalty of not being in wanted range;

binary variable active(beams) determines whether or not to use each beam,
                live(regions) determines if region is sacrificed or not;

free variable totalPenalty total penalties of beams chosen;

equations EQ_totalRad(regions) total amount of radiation delivered to each region,
          EQ_maxNormRad(regions),
          EQ_normalLose,
          EQ_maxBeams,
          EQ_maxWeight(beams),
          EQ_underPenalty(regions),
          EQ_overPenalty(regions),
          EQ_totalPenalty;

* summing weight(beam)*(dose delivered by beam to region) over all beams
EQ_totalRad(regions)..
         totalRad(regions) =e= sum(b, weight(b) * dose(b, regions));

* ensure radiation cap on nonsacrafical regions
EQ_maxNormRad(normalRegions)..
         totalRad(normalRegions) =l= (maxNormRad - 1) * live(normalRegions)
                                   + bigM * (1 - live(normalRegions));

* limit number of sacrificed regions
EQ_normalLose..
         sum(normalRegions, live(normalRegions)) =g= card(normalRegions) - normalLose;

* limit the number of beams that can be used
EQ_maxBeams..
         sum(b, active(b)) =l= maxBeams;

* ensures that each beam used can only be used to a certain weight
EQ_maxWeight(beams)..
         weight(beams) =l= maxWeight * active(beams);

EQ_underPenalty(tumorRegions)..
         penalty(tumorRegions) =g= underPenalty * (minTumorRad - totalRad(tumorRegions));

EQ_overPenalty(tumorRegions)..
         penalty(tumorRegions) =g= overPenalty * (totalRad(tumorRegions) - maxTumorRad);

EQ_totalPenalty..
         totalPenalty =e= sum(tumorRegions, penalty(tumorRegions));

model prob8 /all/;
prob8.optcr = 0;

solve prob8 using mip minimizing totalPenalty;

option weight:0;
display weight.l;
display totalRad.l;
display penalty.l;
display totalPenalty.l;