$title Homework 10 Problem 1
$offuellist offuelxref offsymlist offsymxref
option limrow=0, limcol=0, solprint=off;
*option mip = cplex;

$offlisting
$Ontext

1. Take the first 8 columns of the "wheatdata" matrix (p1*p8) to be predictor
 variables and the last column (p9) to be a response variable. Add a ninth
 predictor variable: a constant shift. Letting X denote the predictor variable
 matrix and y denote the response variable vector, we seek regression
 coefficients b such that Xb approx= y.
   Scale the matrix of predictor variables X so that all its columns have a
 2-norm of 1. In a single GAMS file, calculate coefficients b according to
 different criteria, by solving the following problems. Be sure to use the most
 efficient solver possible in each case (that is, nlp or lp or mip, as
 appropriate).
 a. Least squares: minimize || Xb - y ||22
 b. l-1: minimize || Xb - y ||1 = sum_i | Xi b - yi |.
 c. LASSO: minimize || Xb - y ||2 subject to || b ||1 <= T, where T = 100, 200,
 500. For each value of T, identify the components of b that are (substantially)
 nonzero, then solve a least-squares problem without the constraint
 || b ||1 <= T in which just these components are allowed to be nonzero. (Hint:
 use a loop to avoid replicating solve statements.)
 d. Variable selection: minimize || Xb - y ||1, with at most 3 regression
 coefficients (components of b) taking on nonzero values.
 e. Solve the Huber regression problem with sigma=10, using the dual formulation
 described in class.

 Write out a file called wheat.txt (using "put") that tabulates the following
 for each variant of the regression problem above:
    * the optimal vales of the nine regression coefficients - the vector b;
    * the sum of squares || Xb - y ||22;
    * the l-1 norm || Xb - y ||1
 with an appropriate label for each entry.

$Offtext
$onlisting

set vars  /p1*p9/,
    years /1930*1962/,
    i/1*3/;

alias(years, y1, y2);
alias (vars, v);
*/vars = k; years = n;

scalar shift /1/,
       T,
       maxNonzero /3/,
       bigM just take 5 /10000/,
       sigma /10/;

$include iowheat.dat


parameter X(years, vars),
          y(years),
          norm(vars),
          TValues(i)/1 100, 2 200, 3 500/;

*fill X
loop((years, vars),
         X(years, vars) = wheatdata(years, vars)$(ord(vars) <= 8) +
                          shift$(ord(vars) = 9);
);
*scale X so that the norm of each column to 1
norm(vars) = sqrt(sum(y1, X(y1, vars)**2));
X(years, vars) = X(years, vars) / norm(vars);

*fill y
loop(years,
         y(years) = wheatdata(years, 'p9');
);

*check that norm of all columns is 1
*norm(vars) = sqrt(sum(y1, X(y1, vars)**2));
*display norm;

binary variable s(vars);

free variable b(vars),
              r(years),
              t1(vars),
              w(years),
              error(years),
              totalError;

equations
         EQ_error(years),
         EQ_leastSquares,
         EQ_norm1,
         EQ_l1,
         EQ_l11(years),
         EQ_l12(years),
         EQ_lasso1(vars),
         EQ_lasso2(vars),
         EQ_lasso3,
         EQ_vs1(vars),
         EQ_vs2(vars),
         EQ_vs3,
         EQ_huber1,
         EQ_huber2,
         EQ_huber3,
         EQ_huber4;

EQ_error(years)..
         error(years) =e= sum(v, X(years,v) * b(v)) - y(years);

EQ_leastSquares..
         totalError =e= sum(y1, error(y1)*error(y1));

EQ_l1..
         totalError =e= sum(y1, r(y1));

EQ_l11(years)..
         r(years) =g= error(years);

EQ_l12(years)..
         r(years) =g= -error(years);

EQ_lasso1(vars)..
         b(vars) =g= -t1(vars);

EQ_lasso2(vars)..
         b(vars) =l= t1(vars);

EQ_lasso3..
         sum(v, t1(v)) =l= T;

EQ_vs1(vars)..
         b(vars) =l= bigM * s(vars);
EQ_vs2(vars)..
         b(vars) =g= (-bigM) * s(vars);
EQ_vs3..
         sum(v, s(v)) =l= maxNonZero;

EQ_huber1..
         totalError =e= .5 * sum(y1, w(y1)*w(y1)) + sum(y1, y(y1)*w(y1));

EQ_huber2(vars)..
         sum(y1, X(y1, vars) * w(y1)) =e= 0;

EQ_huber3(years)..
         w(years) =g= -sigma;

EQ_huber4(years)..
         w(years) =l= sigma;

*min (1/2) w^T w  + y^Tw
*s.t. X^Tw = 0,   -sigma <= w_i <= sigma   for i=1,2,...N

*$ontext
model prob10a/EQ_error, EQ_leastSquares/;
prob10a.optcr = 0;
solve prob10a using nlp minimizing totalError;
display b.l;

model prob10b/EQ_error, EQ_l1, EQ_l11, EQ_l12/;
prob10b.optcr = 0;
solve prob10b using lp minimizing totalError;
display b.l;
*$offtext

model prob10c/EQ_error, EQ_leastSquares, EQ_lasso1, EQ_lasso2, EQ_lasso3/;
prob10c.optcr = 0;
loop(i,
*        set proper T value
         T = TValues(i);

         solve prob10c using nlp minimizing totalError;

*        fix least useful b values to 0
         loop(v,
               if(abs(b.l(v)) < 10**(-10),
                       b.l(v) = 0;
                       b.fx(v) = 0;
               );
         );
*        display T;
*        display b.l;

         solve prob10a using nlp minimizing totalError;
         display T;
         display b.l;

*        unconstrain b for next time
         b.lo(vars) = -INF;
         b.up(vars) = INF;
);

totalError.l = 0;
b.l(vars) = 0;
r.l(years) = 0;

model prob10d/EQ_vs1, EQ_vs2, EQ_vs3, EQ_error, EQ_l1, EQ_l11, EQ_l12/;
prob10d.optcr = 0;
solve prob10d using mip minimizing totalError;
display b.l;

model prob10e/EQ_huber1, EQ_huber2, EQ_huber3, EQ_huber4/;
prob10e.optcr = 0;
solve prob10e using nlp minimizing totalError;

loop(vars,
         b.l(vars) = EQ_huber2.m(vars);
);
display b.l;
